<?php
/*
Plugin Name: ContentMonster posts
Plugin URI:  http://contentmonster.ru
Description: Наполняйте ваш блог качественным контентом через ContentMonster.ru.
Author: ContentMonster.ru
Author URI: http://contentmonster.ru
Version: 0.6
*/
/** Настройки **/
define('CM_PLUGIN_UPDATE_URL', 'http://contentmonster.ru/plugins/wp/wordpress_updater'); // url для обновлений
define('CM_PLUGIN_PATH', dirname(__FILE__));
define('CM_PLUGIN_URL', plugins_url('', __FILE__));

// Главный класс плагина
class ContentMonster
{
    public function __construct()
    {
        register_activation_hook(__FILE__, array(&$this, 'activate'));
        add_action('wp_head', array(&$this, 'meta'));
    }

    public function activate()
    {
        add_option('contentmonster_apikey', '');
        add_option('contentmonster_auth_success', 0);

        $this->generate_apikey();
    }

    protected function generate_apikey()
    {
        /*if (get_option('contentmonster_apikey') !== '')
        {
            return;
        }*/

        $key = $this->rand_sha1(40);
        update_option('contentmonster_apikey', $key);
        return $key;
    }

    protected function rand_sha1($length)
    {
        $max = ceil($length / 40);
        $random = '';
        for ($i = 0; $i < $max; $i++) {
            $random .= sha1(microtime(true) . mt_rand(10000, 90000));
        }
        return substr($random, 0, $length);
    }

    function meta()
    {
        global $post;
        if (!is_singular())
            return;

        $description = get_post_meta($post->ID, 'seo_description', TRUE);
        if ($description) {
            echo "<meta name='description' content='{$description}' />\n";
        }
        $keywords = get_post_meta($post->ID, 'seo_keywords', TRUE);
        if ($keywords) {
            echo "<meta name='keywords' content='{$keywords}' />\n";
        }
    }
}

// Класс настроек плагина в админке
class ContentMonsterAdmin extends ContentMonster
{
    public function __construct()
    {
        add_action('admin_menu', array(&$this, 'admin_menu'));
    }

    public function admin_menu()
    {
        add_options_page('ContentMonster posts', 'ContentMonster posts', 'manage_options', 'contentmonster', array(&$this, 'options'));
    }

    public function options()
    {
        // Обновляем API ключ
        if (isset($_POST['updateapikey'])) {
            $this->generate_apikey();
        }

        ?>
    <div class="wrap">
        <h1>ContentMonster posts</h1>
        <?php if (!get_option('contentmonster_auth_success')): ?>
        <p>Сделайте 3 шага, чтобы с легкостью получать качественные посты на ваш сайт. <a
                href="http://contentmonster.ru/wppluginsetup" target="_blank">Подробная инструкция по установке</a></p>
        <br>
        <p>Шаг 1. <a href="http://contentmonster.ru/?r=wordpress" target="_blank">Зарегистрируйтесь</a> на бирже копирайтинга
            ContentMonster.ru и авторизуйтесь на ней.</p>
        <p>Шаг 2. В <a href="http://contentmonster.ru/export/settings" target="_blank">настройках</a> добавьте URL этого сайта, выберите движок WordPress и введите ключ API <strong><?php echo get_option('contentmonster_apikey')?></strong>.
            Нажмите кнопку "Сохранить". </p>
        <p>Шаг 3. <a href="http://contentmonster.ru/order/add" target="_blank">Закажите</a> статью у профессионального копирайтера.
            Когда статья будет готова, с легкостью экспортируйте её на ваш блог.</p>
        <br>
                <form method="post" action="" name="updateapikeyform">
                    <input type="hidden" name="updateapikey" value="1">
                    <p><a href="#" onclick="document.forms['updateapikeyform'].submit();return false;">Изменить ключ
                        API</a></p>
                </form>
            <?php else: ?>
        <p><a href="http://contentmonster.ru/wppluginsetup" target="_blank">FAQ</a>
           <a href="http://contentmonster.ru/ticket" target="_blank">Обратная связь</a></p>
        <br>
        <form method="post" action="">
            <p>Ключ API: <input size="55" type="text" readonly value="<?=get_option('contentmonster_apikey')?>"> <input
                    class="button-primary" name="updateapikey" type="submit" value="Создать новый ключ"></p>
        </form>
        <p><span style="font-size:0.8em">Ключ API нужен, чтобы вы смогли загружать посты с ContentMonster.ru</span></p>
        <?php endif; ?>
    </div>
    <?php
    }
}

if ( ! class_exists('IXR_Server'))
{
    include_once(ABSPATH . WPINC . '/class-IXR.php');
}

if ( ! class_exists('wp_xmlrpc_server'))
{
    include_once(ABSPATH . WPINC . '/class-wp-xmlrpc-server.php');
}


class ContentMonster_XMLRPC extends wp_xmlrpc_server
{
    public function __construct()
    {
        parent::__construct();

        $methods = array(
            'contentmonster.checkKey' => 'this:contentmonster_checkKey',
            'contentmonster.uploadPost' => 'this:contentmonster_uploadPost',
            'contentmonster.getCategories' => 'this:contentmonster_getCategories',
            'contentmonster.getTags' => 'this:contentmonster_getTags'
        );

        $this->methods = array_merge($this->methods, $methods);
    }

    public static function contentmonster_getName()
    {
        return __CLASS__;
    }

    // Переопределяем метод IXR_Server::call для последовательной передачи аргументов
    function call($methodname, $args)
    {
        if (!$this->hasMethod($methodname)) {
            return new IXR_Error(-32601, 'server error. requested method ' . $methodname . ' does not exist.');
        }
        $method = $this->callbacks[$methodname];

        // Are we dealing with a function or a method?
        if (is_string($method) && substr($method, 0, 5) == 'this:') {
            // It's a class method - check it exists
            $method = substr($method, 5);
            if (!method_exists($this, $method)) {
                return new IXR_Error(-32601, 'server error. requested class method "' . $method . '" does not exist.');
            }

            //Call the method
            $result = call_user_func_array(array($this, $method), $args);
        }

        return $result;
    }

    public function contentmonster_checkKey($key)
    {
        if ($key == get_option('contentmonster_apikey')) {
            update_option('contentmonster_auth_success', 1);
            return 1;
        } else {
            return 0;
        }
    }

    public function contentmonster_uploadPost($key, $post)
    {
        if (!$this->contentmonster_checkKey($key))
            return 0;

        require_once(ABSPATH . 'wp-admin/includes/image.php');

        $post_data = array('post_status' => 'publish', 'post_author' => 1);

        $post_data['post_title'] = $post['title'];
        $post_data['post_content'] = $post['content'];
        $post_data['post_category'] = is_array($post['category_id']) ? $post['category_id'] : array($post['category_id']);
        $post_data['tags_input'] = $post['tags'];
        if (!empty($post['date'])) {
            $post_data['post_date'] = $post['date'];
        }



        if (!empty($post['status']) && in_array($post['status'], array('publish', 'draft'))) {
            $post_data['post_status'] = $post['status'];
        }

        $post_id = wp_insert_post($post_data);
        if (empty($post_id)) return 0;
        if (!empty($post['seo_description'])) {
            update_post_meta($post_id, 'seo_description', $post['seo_description']);
        }

        if (!empty($post['seo_keywords'])) {
            update_post_meta($post_id, 'seo_keywords', $post['seo_keywords']);
        }
        add_post_meta($post_id, '_aioseop_disable', 1);

        $imghost = 'http://contentmonster.ru/';
        $pattern = '`src\s*=\s*(?:"|\')?' . $imghost . '([/0-9a-z.]+)`i';
        if (preg_match_all($pattern, $post_data['post_content'], $matches)) {
            $arrsource = array_unique($matches[1]);
            $update_post_data = array('ID' => $post_id, 'post_content' => $post_data['post_content']);
            foreach ($arrsource as $imageId) {
                $url = $imghost . $imageId;
                $ret = media_sideload_image($url, $post_id);
                if (is_string($ret)) {
                    preg_match('`(?<=src=(?:"|\')).+(?="|\')`iU', $ret, $src);
                    $update_post_data['post_content'] = str_ireplace($url, $src[0], $update_post_data['post_content']);
                }
            }
            wp_update_post($update_post_data);
        }
        return $post_id;
    }

    public function contentmonster_getTags($key)
    {
        if (!$this->contentmonster_checkKey($key))
            return 0;

        $tags_data = get_tags(array(
            'hide_empty' => FALSE,
        ));

        $tags = array();
        foreach ($tags_data as $tag) {
            $tags[$tag->term_id] = $tag->name;
        }

        return $tags;
    }

    public function contentmonster_getCategories($key)
    {
        if (!$this->contentmonster_checkKey($key))
            return 0;

        $categories_data = get_categories(array(
            'hide_empty' => FALSE
        ));

        $categories = array();
        foreach ($categories_data as $category) {
            $categories[$category->term_id] = $category->name;
        }

        return $categories;
    }
}


add_filter('wp_xmlrpc_server_class', array('ContentMonster_XMLRPC', 'contentmonster_getName'));


class WpPluginAutoUpdate
{
    # URL to check for updates, this is where the index.php script goes
    public $api_url;

    # Type of package to be updated
    public $package_type;

    public $plugin_slug;
    public $plugin_file;

    public function WpPluginAutoUpdate($api_url, $type, $slug)
    {
        $this->api_url = $api_url;
        $this->package_type = $type;
        $this->plugin_slug = $slug;
        $this->plugin_file = $slug . '/' . $slug . '.php';
    }

    public function check_for_plugin_update()
    {
        $args = func_get_args();

        $checked_data = $args[0];

        if (empty($checked_data->checked))
            return $checked_data;

        $request_args = array(
            'slug' => $this->plugin_slug,
            'version' => $checked_data->checked[$this->plugin_file],
            'package_type' => $this->package_type,
        );

        $request_string = $this->prepare_request('basic_check', $request_args);

        // Start checking for an update
        $raw_response = wp_remote_post($this->api_url, $request_string);

        if (!is_wp_error($raw_response) && ($raw_response['response']['code'] == 200)) {
            $response = unserialize($raw_response['body']);

            if (is_object($response) && !empty($response)) // Feed the update data into WP updater
                $checked_data->response[$this->plugin_file] = $response;
        }

        return $checked_data;
    }

    // используется для фильтра plugins_api, $def - всегда false
    public function plugins_api_call($def, $action, $args)
    {
        if ($args->slug != $this->plugin_slug)
            return false;

        // Get the current version
        $plugin_info = get_site_transient('update_plugins');
        $current_version = $plugin_info->checked[$this->plugin_file];
        $args->version = $current_version;
        $args->package_type = $this->package_type;

        $request_string = $this->prepare_request($action, $args);

        $request = wp_remote_post($this->api_url, $request_string);

        if (is_wp_error($request)) {
            $res = new WP_Error('plugins_api_failed', __('An Unexpected HTTP Error occurred during the API request.</p> <p><a href="?" onclick="document.location.reload(); return false;">Try again</a>'), $request->get_error_message());
        } else {
            $res = unserialize($request['body']);

            if ($res === false)
                $res = new WP_Error('plugins_api_failed', __('An unknown error occurred'), $request['body']);
        }

        return $res;
    }

    public function prepare_request($action, $args)
    {
        global $wp_version;

        $site_url = site_url();

        $wp_info = array(
            'site-url' => $site_url,
            'version' => $wp_version, // версия WP, глобальная
        );

        return array(
            'body' => array(
                'action' => $action, 'request' => serialize($args),
                'wp-info' => serialize($wp_info),
            )
        );
    }
}


function ContentMonsterPluginInit()
{
    $wp_plugin_auto_update = new WpPluginAutoUpdate(CM_PLUGIN_UPDATE_URL, 'stable', basename(dirname(__FILE__)));
    // Take over the update check
    add_filter('pre_set_site_transient_update_plugins', array($wp_plugin_auto_update, 'check_for_plugin_update'));
    // Take over the Plugin info screen
    add_filter('plugins_api', array($wp_plugin_auto_update, 'plugins_api_call'), 10, 3);
    $ContentMonster = new ContentMonster();
    if (is_admin()) {
        $ContentMonsterAdmin = new ContentMonsterAdmin();
    }
}

ContentMonsterPluginInit();
